/* FOLD info
        ~~~~ */

/* ARexxPort.e 1.0 - par Leon Woestenberg (leon@stack.urc.tue.nl) */
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
/*

   Je prsente ici quelques fonctions de base pour rendre vos
   programmes E capable de communiquer avec d'autres programmes
   multitaches via AREXX. Avec ces fonctions vos programmes
   pourront:

    Recevoir et ragir aux commandes ARexx qui ont t envoy.
    Appeler Rexx Master pour lancer des commandes que votre programme
     ne comprend pas. Ce peut tre des script AREXX (dans REXX:) aussi.
    Envoyer des commandes AREXX  d'autres programmes ou  Rxx Master.

   Ce source est du domaine public alors utilisez le :)

   Le source est abondamment commet et s'explqie quasiment par lui-mme.
   La chse la plus dur est : Quand un message AREXX arrivant contient
   une commande qui n'est pas support par votre programme, ce message
   n'est pas renvoy DE SUITE. D'abord, un nouveau message est cr et
   envoy  Rexx Master. On peut enlever  le premier message de la mmoire
   (en utilisant le pointeur). De cette faon, les scripts AREXX peuvent
   tre utilis avec des commandes AREXX. Notez que l'extension par dfaut
   de ces scripts peut tre adapt  votre programme dans la procdure
   sendRexxMsg.

   ARexx ajoute une grande possibilit  votre Amiga multitache, comme
   des taches qui interagissent en temps rel entre eux, comme a il peuvent
   utiliser les caractristique des autres. Pensez au possibilit offerte 
   vos programmes tendus avec une interface ARexx.

   Si vous avez des questions, suggestions, ou report d'erreurs, joignez moi
   via Internet. Les questions/discussions gnrale sur ce source sont
   aussi les bienvenues sur l'Amiga E mailing list.

                              Leon Woestenberg (leon@stack.urc.tue.nl)

*/
/* FEND */
/* FOLD "modules" */

MODULE 'exec/ports','exec/nodes'
MODULE 'rexxsyslib','rexx/rexxio','rexx/rxslib','rexx/errors','rexx/storage'
MODULE 'dos/dos'

/* FEND */
/* FOLD "definitions" */

DEF hostport=NIL:PTR TO mp
DEF runflag=TRUE
DEF unconfirmed=0

/* FEND */

/* FOLD "main" */
PROC main() HANDLE

  /* open rexx library */
  IF (rexxsysbase:=OpenLibrary('rexxsyslib.library',0))

    /* cr un port hte pour arexx */
    IF (hostport:=createPort('HoteExemple',0))

    /* exemples de votre proramme envoyant des cxommandes arexx  d'autres
       ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

    /*

    /* Dit  GoldED de dplacer le curseur d'une ligne vers le haut */
    sendRexxMsg('GOLDED.1','UP',NIL,0)

    /* dit au shell d'xcuter la commande list */
    sendRexxMsg('REXX','"ADDRESS COMMAND list"',NIL,0)

    /* lance un script depuis REXX: (mettez l'extension par dfaut dans sendRexxMsg) */
    sendRexxMsg('REXX','scriptname',NIL,0)

    /* envoie une commande 'QUIT'  nous mme :o) */
    sendRexxMsg('HoteExemple','QUIT',NIL,0)

    */

    /* informe l'utilisateur que a marche */
    WriteF('Attend des vnements. Envoyez moi une commande arexx QUIT en tapant:\nrx "ADDRESS \aHoteExemple\a; QUIT" depuis un shell. Ou pressez CTRL-C pour m\aarrter.\n')

    /* attend */
    wait()

    deletePort(hostport)
    ELSE
      WriteF('Dsol bonhomme, je crois que je tourne dja une fois.\n')
    ENDIF
    CloseLibrary(rexxsysbase)
  ELSE
    WriteF('Ne peut pas ouvrir la rexxsyslib.library. J\aen ai vraiment besoin!\n')
  ENDIF
EXCEPT
  WriteF('Quleque chose comme \d n'a pas march, tu vois?\n')
  deletePort(hostport)
  hostport:=NIL
  IF rexxsysbase
    CloseLibrary(rexxsysbase)
    rexxsysbase:=NIL
  ENDIF
ENDPROC
/* FEND */
/* FOLD "wait" */
PROC wait()

  DEF signalmask=0
  DEF hostmask=0

  /* masque signal pour notre port hte */
  hostmask:=Shl(1,hostport.sigbit)

  /* continue  tourner ou messages non confirms? */
  WHILE runflag OR unconfirmed

    /* attend les signaux d'vnements */
    signalmask:=Wait(hostmask OR SIGBREAKF_CTRL_C)

    /* et gre les vnements qui arrive */
    IF signalmask AND hostmask THEN handleRexxMsg()
    IF signalmask AND SIGBREAKF_CTRL_C THEN runflag:=FALSE

  ENDWHILE
ENDPROC
/* FEND */

/* FOLD "handleRexxMsg" */
PROC handleRexxMsg()

  /* pointeur sur le message  grer */
  DEF rexxmsg:PTR TO rexxmsg
  /* pointeur sur messagenode */
  DEF msgnode:PTR TO mn
  /* pointeur sur la liste de noeuds du message */
  DEF listnode:PTR TO ln

  /* liste de 16 pointers sur les chaines de commande */
  DEF rexxargs:PTR TO LONG

  /* pointe sur le premier caractre de la commande */
  DEF command:PTR TO CHAR

  /* (un autre) message  la queue? */
  WHILE rexxmsg:=GetMsg(hostport)

    /* met le pointeur sur messagenode */
    msgnode:=rexxmsg.mn

    /* met le pointeur sur la liste de noeuds */
    listnode:=msgnode.ln

    /* met le pointeur sur les commands */
    rexxargs:=rexxmsg.args

    /* rponse de confirmation d'un message envoy par nous? */
    IF listnode.type=NT_REPLYMSG

      /* original message pointer present? */
      IF rexxargs[15]
        /* rpond au message original */
        ReplyMsg(rexxargs[15])
      ENDIF

      /* efface ce message de confirmation */
      DeleteArgstring(rexxargs[0])
      DeleteRexxMsg(rexxmsg)

      /* diminue le compteur non-confirm */
      DEC unconfirmed

    /* un tout nouveau message */
    ELSE

      /* pointe sur la commande aprs avoir sauts les espaces etc. */
      command:=TrimStr(rexxargs[0])
      WriteF('We received an ARexx command: \s\n',command)

      /* initialise les codes sortants */
      rexxmsg.result1:=0
      rexxmsg.result2:=NIL

      /* exemple de gestion d'une commande que quequ'un nous a envoy
         ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

      /* command known? */
      IF StrCmp('QUIT',command,ALL)
        WriteF('So let\as quit here.\n')
        runflag:=FALSE
        ReplyMsg(rexxmsg)

      /* commande inconnue */
      ELSE
        WriteF('Hmmm. Commance inconnue. On l'envoie  Rexx Master...\n')

        /* on fait suivre cette commande  rexx master (peut-tre c'est un script?) */
        /* le message original n'est pas rpondu pour le moment */
        /* tant qu'on n'a pas eu confirmation de rexx master. */
        /* Pour cela, on garde ce message dans un pointeur de message rexxargs[15] */
        /* in rexxargs[15] de la commande envoye  Rexx Master */

        /* message ne peut pas tre envoy  rexx? */
        IF sendRexxMsg('REXX',rexxargs[0],rexxmsg,0)=NIL

          /* met le code de retour : erreur fatale */
          xReplyRexxCmd(rexxmsg,RC_FATAL,NIL)

          /* rpond au message */
          ReplyMsg(rexxmsg)

        ENDIF
      ENDIF
    ENDIF
  ENDWHILE
ENDPROC
/* FEND */
/* FOLD "sendRexxMsg(hostname,command,unknownmsg,flags)" */
PROC sendRexxMsg(hostname,command,unknownmsg,flags)

  DEF arexxport=NIL:PTR TO mp
  DEF rexxmsg=NIL:PTR TO rexxmsg
  DEF rexxargs:PTR TO LONG
  DEF listnode=NIL:PTR TO ln
  DEF temp=NIL

  /* retourne si le port hte n'est pas prsent */
  IF hostport=NIL THEN RETURN NIL

  listnode:=hostport.ln

  /* retourne si on ne pas faire un message rexx */
  IF (rexxmsg:=CreateRexxMsg(hostport,'rexx',listnode.name))=NIL THEN RETURN NIL

  /* pointeur vers les commandes */
  rexxargs:=rexxmsg.args

  /* peut-on crer un argstring? */
  IF temp:=CreateArgstring(command,StrLen(command))

    /* met le premier argstring */
    rexxargs[0]:=temp

    /* met les flags */
    rexxmsg.action:=RXCOMM OR flags

    /* met le pointeur message original dans un pointeur chaine de 16 */
    rexxargs[15]:=unknownmsg

    /* interdit le multitache */
    Forbid()

    /* envoie notre message vers un port dja existant */
    IF (arexxport:=FindPort(hostname)) THEN PutMsg(arexxport,rexxmsg)

    /* permet le multitache */
    Permit()

    /* sended? */
    IF arexxport

      /* augmente le compteur de non-confirm */
      INC unconfirmed

      /* messages envoys avec succs */
      RETURN rexxmsg
    ENDIF
  ENDIF

  IF temp
    DeleteArgstring(temp)
  ENDIF
  IF rexxmsg
    DeleteRexxMsg(rexxmsg)
  ENDIF
  RETURN NIL
ENDPROC
/* FEND */
/* FOLD "replyRexxMsg(rexxmsg,rc,returnstring)" */
PROC xReplyRexxCmd(rexxmsg:PTR TO rexxmsg,rc,returnstring)

  /* met le code de retour */
  rexxmsg.result1:=rc

  /* et un pointeur sur la chaine sortante */
  rexxmsg.result2:=IF (rexxmsg.action AND RXFF_RESULT) AND (returnstring<>NIL) THEN CreateArgstring(returnstring,StrLen(returnstring)) ELSE NIL

ENDPROC
/* FEND */
/* FOLD "createPort(portname,priority)" */
PROC createPort(portname,priority)

  DEF port=NIL:PTR TO mp
  DEF node=NIL:PTR TO ln

  /* met le port public? */
  IF portname

    /* personne ne fait le mme port SVP */
    Forbid()

    /* notre port va-t-il tre unique ? */
    IF FindPort(portname)=0

      /* peut-on faire un port? */
      IF port:=CreateMsgPort()

        node:=port.ln

        /* rempli le nom */
        node.name:=portname

        /* priorit du port public */
        node.pri:=priority

        /* et fat ce port public */
        AddPort(port)
      ENDIF
    ENDIF

    /* multitache */
    Permit()

  /* fait juste un port priv */
  ELSE

    /* essaie de faire un port */
    port:=CreateMsgPort()

  ENDIF
/* retourne le pointeur au port, ou NIL si le port ne peut tre fait (unique) */
ENDPROC port
/* FEND */
/* FOLD "deletePort(port)" */
PROC deletePort(port:PTR TO mp)

  DEF node=NIL:PTR TO ln
  DEF msg=NIL:PTR TO mn

  /* pointeur donn ? */
  IF port

    node:=port.ln

    /* si public alors enlve de la liste public du port */
    IF node.name THEN RemPort(port)

    /* plus de message SVP */
    Forbid()

    /* enlve tous les messages  la queue */
    WHILE msg:=GetMsg(port) DO ReplyMsg(msg)

    /* efface le port */
    DeleteMsgPort(port)

    /* multitache */
    Permit()
  ENDIF
ENDPROC
/* FEND */

